<?php
/*
 * plugin ChamPlus
 * 
 * ré-écriture complète de champArt.
 * Nécessite Pluxml 5.4, HTML5, PHP 5.6
 * 
 * déplacement de la feuille de styles dans le dossier css et renommée admin.css. suppression du hook AdminTopEndHead
 * les paramètres champ, type et groupe sont renommés en name, textarea et group
 * le nouveau paramètre textarea est numérique (valeur 0 ou 1)
 * suppression des entités HTML dans le fichier de lang (UTF-8 !!!)
 * utilisation des attributs placehorder et required pour les balises <input type="text" />
 * suppression du hook AdminArticleTop
 * on peut préciser une chaîne de format pour afficher les champs (mot-clés #name#, #label#, #value#, #group")
 * suppression du fichier d'aide. Voir aide dans panneau de config.
 * 
 * Pas de textarea dans les pages statiques !!
 * */

define('CHAMPLUS_PREFIX', 'cps_');

class chamPlus extends plxPlugin {
	
	public $paramsNames = array('name', 'label', 'textarea', 'group', 'static');

	public function __construct($default_lang) {
		parent::__construct($default_lang);
		
		$this->setConfigProfil(PROFIL_ADMIN);

		/* ******** hooks inside article.php ****************** */
		$this->addHook('AdminArticleSidebar', 'AdminArticleSidebar');
		$this->addHook('AdminArticleContent', 'AdminArticleContent');
		$this->addHook('AdminArticleInitData', 'AdminArticleInitData');
		$this->addHook('AdminArticlePreview', 'AdminArticlePreview');
		$this->addHook('AdminArticlePostData', 'AdminArticlePostData');
		$this->addHook('AdminArticleParseData', 'AdminArticleParseData');
		
		/* ******** hooks inside statique.php ***************** */
		$this->addHook('AdminStatic', 'AdminStatic');
		
		/* ******** hooks inside class.plx.admin.php ********** */
		$this->addHook('plxAdminEditArticleXml', 'plxAdminEditArticleXml');
		$this->addHook('plxAdminEditStatique', 'plxAdminEditStatique');
		$this->addHook('plxAdminEditStatiquesUpdate', 'plxAdminEditStatiquesUpdate');
		$this->addHook('plxAdminEditStatiquesXml', 'plxAdminEditStatiquesXml');
	
		/* ********** hooks inside class.plx.motor.php ******** */
		$this->addHook('plxMotorParseArticle', 'plxMotorParseArticle');
		$this->addHook('plxMotorGetStatiques', 'plxMotorGetStatiques');

		/* ********** new hooks for this plugin ********* */
		// Hook du plugin à utiliser sur le site dans un thème
		$this->addHook('chamPlus', 'chamPlus');
		// renvoie tous les champs sous forme de tableau
		$this->addHook('chamPlusList', 'chamPlusList');

	}

	// pour la saisie des champs dans config.php
	public function isBoolean($name) {
		return in_array($name, array('textarea', 'static'));
	}
	
	// retourne les indices des champs
	public function indices() {
		$buf = array_filter(
			$this->getParams(),
			function($k) { return (strpos($k, 'name') === 0); },
			ARRAY_FILTER_USE_KEY
		);
		return array_map(function($v) { return substr($v, strlen('name')); }, array_keys($buf));
	}
	
	// the field is only for static pages
	private function isStatic($indice) {
		return ($this->getParam('static'.$indice) > 0);
	}

	private function isTextarea($indice) {
		return (! $this->isStatic($indice) and $this->getParam('textarea'.$indice) > 0);
	}

	// ajoute les champs supplémentaires dans le panneau latéral droit de l'édition de l'article
	public function AdminArticleSidebar() {
		$code = '';
		foreach ($this->indices() as $key) {
			if (! $this->isStatic($key) and ! $this->isTextarea($key)) {
				$fieldName = CHAMPLUS_PREFIX.$this->getParam('name'.$key);
				$field = "\$$fieldName";
				$label = ucfirst($this->getParam('label'.$key));
				$code .= <<< EOT
					<div>
						<label for="id_$fieldName">$label&nbsp;:</label>
						<?php plxUtils::printInput('$fieldName',plxUtils::strCheck($field)); echo "\n"; ?>
					</div>

EOT;
			}
		}
		echo $code;
	}

	// ajoute les champs supplémentaires sous le contenu de l'article avec la balise textarea dans l'édition de l'article
	public function AdminArticleContent() {
		$code = '';
		foreach ($this->indices() as $key) {
			if ($this->isTextarea($key)) {
				$fieldName = CHAMPLUS_PREFIX.$this->getParam('name'.$key);
				$field = "\$chamPlus['$fieldName']";
				$label = ucfirst($this->getParam('label'.$key));
				$display = L_ARTICLE_CHAPO_DISPLAY;
				$hide = L_ARTICLE_CHAPO_HIDE;
				$code .= <<< EOT
					<div>
						<label for="id_$fieldName">$label&nbsp;:&nbsp;<a id="toggler_$fieldName" href="javascript:void(0)" onclick="toggleDiv('toggle_$fieldName', 'toggler_$fieldName', '$display','$hide')"><?php echo \$$fieldName==''?'$display':'$hide' ?></a></label>
						<div id="toggle_$fieldName"<?php echo \$$fieldName!=''?'':' style="display:none"' ?>>
						<?php plxUtils::printArea('$fieldName',plxUtils::strCheck(\$$fieldName),35,8,false,'full-width'); ?>
						</div>
					</div>
EOT;
			}
		}
		echo $code;
	}

	// initialise les champs supplémentaires pour un nouvel article
	public function AdminArticleInitData() {
		echo "<?php\n";
		foreach ($this->indices() as $key) {
			if (! $this->isStatic($key)) {
				$fieldName = CHAMPLUS_PREFIX.$this->getParam('name'.$key);
				$field = "\$chamPlus['$fieldName']";
				$label = $this->getParam('label'.$key);
				echo "$field = ''\n";
			}
		}
		echo "?>\n";
	}

	public function AdminArticlePreview() {
		$code = "<?php\n";
		foreach ($this->indices() as $key) {
			if (! $this->isStatic($key)) {
				$fieldName = CHAMPLUS_PREFIX.$this->getParam('name'.$key);
				$code .= <<< RECORD
	\$art['$fieldName'] = \$_POST['$fieldName'];

RECORD;
				}
			}
		$code .= "?>\n";
		echo $code;
	}

	public function AdminArticlePostData() {
		$code = "<?php\n";
		foreach ($this->indices() as $key) {
			if (! $this->isStatic($key)) {
				$fieldName = CHAMPLUS_PREFIX.$this->getParam('name'.$key);
				$code .= <<< RECORD
	\$$fieldName = \$_POST['$fieldName'];

RECORD;
			}
		}
		$code .= "?>\n";
		echo $code;
	}

	public function AdminArticleParseData() {
		$code = "<?php\n";
		foreach ($this->indices() as $key) {
			if (! $this->isStatic($key)) {
				$fieldName = CHAMPLUS_PREFIX.$this->getParam('name'.$key);
				$code .= <<< RECORD
	\$$fieldName = \$result['$fieldName'];

RECORD;
				}
		}
		$code .= "?>\n";
		echo $code;
	}

	// ajoute des champs supplémentaires dans l'édition de la page statique dans statique.php
	public function AdminStatic() {
		// bug Pluxml 5.4: Pas de Hook dans le bloc "elseif (! empty($_GET['p'])) {...}"
		$code = '';
		foreach ($this->indices() as $key) {
			if ($this->isStatic($key)) {
				$fieldName = CHAMPLUS_PREFIX.$this->getParam('name'.$key);
				$label = ucfirst($this->getParam('label'.$key));
				$code .= <<< RECORD
			<div>
				<label for="id_$fieldName">$label</label>
				<?php plxUtils::printInput('$fieldName',plxUtils::strCheck(\$plxAdmin->aStats[\$id]['$fieldName']),'text','50-255'); echo "\n"; ?>
			</div>

RECORD;
			}
		}
		echo $code;
	}

	// Load the fields of article from XML file
	public function plxMotorParseArticle() {
		$code = "<?php\n";
		foreach ($this->indices() as $key) {
			if (! $this->isStatic($key)) {
				$fieldName = CHAMPLUS_PREFIX.$this->getParam('name'.$key);
				$code .= <<< RECORD
	\$art['$fieldName'] = (isset(\$iTags['$fieldName'])) ? plxUtils::getValue(\$values[\$iTags['$fieldName'][0]]['value']) : '';

RECORD;
			}
		}
		$code .= "?>\n";
		echo $code;
	}

	// load data from statiques.xml in class.plx.motor
	public function plxMotorGetStatiques() {
		$code = "<?php\n";
		foreach ($this->indices() as $key) {
			if ($this->isStatic($key)) {
				$fieldName = CHAMPLUS_PREFIX.$this->getParam('name'.$key);
				$code .= <<< RECORD
	\$this->aStats[\$number]['$fieldName']=plxUtils::getValue(\$values[\$iTags['$fieldName'][\$i]]['value']);

RECORD;
			}
		}
		$code .= "?>\n";
		echo $code;
	}

	// Save the  fields of article to XML file
	public function plxAdminEditArticleXml() {
		$code = "<?php\n";
		foreach ($this->indices() as $key) {
			if (! $this->isStatic($key)) {
				$fieldName = CHAMPLUS_PREFIX.$this->getParam('name'.$key);
				$code .= <<< RECORD
	\$xml .= "\t<$fieldName><![CDATA[".plxUtils::cdataCheck(trim(\$content['$fieldName']))."]]></$fieldName>\n";

RECORD;
			}
		}
		$code .= "?>\n";
		echo $code;
	}

	public function plxAdminEditStatique() {
		$code = "<?php\n";
		foreach ($this->indices() as $key) {
			if ($this->isStatic($key)) {
				$fieldName = CHAMPLUS_PREFIX.$this->getParam('name'.$key);
				$code .= <<< RECORD
		\$this->aStats[\$content['id']]['$fieldName'] = \$content['$fieldName'];

RECORD;
				}
		}
		$code .= "?>\n";
		echo $code;
	}
	
	public function plxAdminEditStatiquesUpdate() {
		$code = "<?php\n";
		foreach ($this->indices() as $key) {
			if ($this->isStatic($key)) {
				$fieldName = CHAMPLUS_PREFIX.$this->getParam('name'.$key);
				$code .= <<< RECORD
	\$this->aStats[\$static_id]['$fieldName'] = (isset(\$this->aStats[\$static_id]['$fieldName']) ? \$this->aStats[\$static_id]['$fieldName'] : '');

RECORD;
				}
		}
		$code .= "?>\n";
		echo $code;
	}

	public function plxAdminEditStatiquesXml() {
		$code = "<?php\n";
		foreach ($this->indices() as $key) {
			if ($this->isStatic($key)) {
				$fieldName = CHAMPLUS_PREFIX.$this->getParam('name'.$key);
				$code .= <<< RECORD
	\$xml .= "<$fieldName><![CDATA[".plxUtils::cdataCheck(\$static['$fieldName'])."]]></$fieldName>";

RECORD;
			}
		}
		$code .= "?>\n";
		echo $code;
	}

	public function chamPlus($params) {
		global $plxMotor;
		
		if (is_string($params)) {
			$name = $params;
			$format = false;
		} else
			list($name, $format) = array_pad($params, 2, false);
		$nameField = CHAMPLUS_PREFIX.$name;
		if ($plxMotor->mode == 'static') {
			$static_id =  $plxMotor->cible;
			$value = plxUtils::strCheck($plxMotor->aStats[$static_id][$nameField]);
		}
		else
			$value = $plxMotor->plxRecord_arts->f($nameField);

		if ($format === true)
			// pas d'affichage, on retourne simplement la valeur
			return $value;
		else if ($format === false)
				// on affiche uniquement la valeur
				echo $value;
		else {
			$label = '';
			$group = '';
			foreach($this->indices() as $idx) {
				if ($this->getParam('name'.$idx) == $name) {
					$label = $this->getParam('label'.$idx);
					$group = $this->getParam('group'.$idx);
					break;
				}
			}
			$patterns = array('#name#', '#value#', '#label#', '#group#');
			$replaces = array($name, $value, $label, $group);					
			echo str_replace($patterns, $replaces, $format);
		}
		return $false;
	}

	public function chamPlusList() {
		global $plxMotor;
		
		$content = array();
		foreach($this->indices() as $idx) {
			$name = $this->getParam('name'.$idx);
			$nameField = CHAMPLUS_PREFIX.$name;
			if (($plxMotor->mode == 'static') and $this->isStatic($idx)) {
				$static_id =  $plxMotor->cible;
				$value = addslashes(plxUtils::strCheck($plxMotor->aStats[$static_id][$nameField]));
			}
			elseif (($plxMotor->mode != 'static') and ! $this->isStatic($idx))
				$value = addslashes($plxMotor->plxRecord_arts->f($nameField));
			else
				$value = false;
			if ($value !== false) {
				$label = addslashes($this->getParam('label'.$idx));
				$group = addslashes($this->getParam('group'.$idx));
				$content[] = "'$name'=>array('label'=>'$label', 'value'=>'$value', 'group'=>'$group')";
			}
		}
		
		if (empty($content))
			$code = '<?php return false; ?>';
		else {
			$a = implode(",", $content);
			$code = "\$a = array($a); return \$a;";
		}
		return $code;
	}

}
?>
